#include "select_server.h"
#include "utl.h"

select_server::select_server( const short port)
{
	memset( &server_addr, 0, sizeof(server_addr) );

	server_addr.sin_family = AF_INET;
	server_addr.sin_port   = htons(port);
	server_addr.sin_addr.s_addr = htonl( INADDR_ANY );
}

select_server::~select_server()
{
	close(sock_serv);
}

int select_server::initServer()
{
	sock_serv = socket(AF_INET, SOCK_STREAM, 0);
	if(sock_serv < 0){
		perror("socket error!\n");
		return -1;
	}

	int on = 1;
	setsockopt( sock_serv, SOL_SOCKET, SO_REUSEADDR, &on, sizeof(on) );

	int flag = bind(sock_serv, (struct sockaddr*)&server_addr, sizeof(server_addr));
	if(flag < 0){
		perror("bind error!\n");
		close(sock_serv);
		return -1;
	}
	return 0;
}

void select_server::starListen()
{
	int flag = listen(sock_serv, 5);
	if(flag < 0){
		perror("listen error!\n");
		close(sock_serv);
		return;
	}
	printf("listen!\n");
	fd_set client_fdset;	//监控文件描述符集合
	int maxsock;            //监控文件描述符中最大的文件号
	struct timeval tv;		//超时返回时间
	int client_sockfd[5];   //存放活动的sockfd
	bzero((void*)client_sockfd,sizeof(client_sockfd));
	int conn_amount = 0;    //用来记录描述符数量
	maxsock = sock_serv;

	char buffer[1024];

	while(1){

		//初始化文件描述符号到集合
		FD_ZERO(&client_fdset);
		//加入服务器描述符
		FD_SET(sock_serv,&client_fdset);

		//设置超时时间
		tv.tv_sec = 30; //30秒
		tv.tv_usec = 0;

		//把活动的句柄加入到文件描述符中
		for(int i = 0; i < 5; ++i){//程序中Listen中参数设为5,故i必须小于5
			if(client_sockfd[i] != 0){
				FD_SET(client_sockfd[i], &client_fdset);
			}
		}
		//printf("put sockfd in fdset!\n");
		//select函数
		flag = select(maxsock+1, &client_fdset, NULL, NULL, &tv);
		if(flag < 0){
			perror("select error!\n");
			break;
		}
		else if(flag == 0){
			printf("timeout!\n");
			continue;
		}

		//轮询各个文件描述符
		for(int i = 0; i < conn_amount; ++i){
			//FD_ISSET检查client_sockfd是否可读写，>0可读写
			if(FD_ISSET(client_sockfd[i], &client_fdset)){
				printf("start recv from client[%d]:\n",i);
				flag = recv(client_sockfd[i], buffer, 1024, 0);
				if(flag <= 0){
					printf("client[%d] close\n", i);
					close(client_sockfd[i]);
					FD_CLR(client_sockfd[i], &client_fdset);
					client_sockfd[i] = 0;
				}
				else{
					printf("recv from client[%d] :%s\n", i, buffer);
				}
			}
		}

		//检查是否有新的连接，如果收，接收连接，加入到client_sockfd中
		if(FD_ISSET(sock_serv, &client_fdset)){
			//接受连接
			struct sockaddr_in client_addr;
			size_t size = sizeof(struct sockaddr_in);
			int sock_client = accept(sock_serv, (struct sockaddr*)(&client_addr), (unsigned int*)(&size));
			if(sock_client < 0){
				perror("accept error!\n");
				continue;
			}
			//把连接加入到文件描述符集合中
			if(conn_amount < 5){
				client_sockfd[conn_amount++] = sock_client;

				bzero(buffer,1024);
				strcpy(buffer, "this is server! welcome!\n");
				send(sock_client, buffer, 1024, 0);

				printf("new connection client[%d] %s:%d\n", conn_amount, inet_ntoa(client_addr.sin_addr), ntohs(client_addr.sin_port));

				bzero(buffer,sizeof(buffer));
				flag = recv(sock_client, buffer, 1024, 0);
				if(flag < 0){
					perror("recv error!\n");
					close(sock_serv);
					return;
				}
				printf("recv : %s\n",buffer);

				if(sock_client > maxsock){
					maxsock = sock_client;
				}
				else{
					printf("max connections!!!quit!!\n");
					break;
				}
			}
		}
	}

	for(int i = 0; i < 5; ++i){
		if(client_sockfd[i] != 0){
			close(client_sockfd[i]);
		}
	}
}
